/*  autorefresh - autorefresh for output.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade.h>
#include <string.h>
#include <time.h>

#include "file_selection.h"
#include "refresh.h"
#include "interface.h"

/* interface */
extern main_window interface;

/* autorefresh */
extern gchar *filename;

static guint time_step;
static guint cicle = 0;
static gdouble spin1 = 1; /* start */
static gdouble spin2 = 1; /* stop */
static gboolean state_save;
static gchar *filename_copy;

guint timeout_id = 0;


/* these 2 functions require to save single autorefresh-output */
static void
autorefresh_save_confirm_response (GtkWidget   *dialog, int response_id, GtkWindow   *transient_parent)
{
	if (response_id == GTK_RESPONSE_YES)
		state_save = TRUE;
}

static gboolean 
control_confirm (void)
{
	GtkWidget *dialog;
	
	state_save = FALSE;

	/* save: YES or NO */
	dialog = gtk_message_dialog_new (NULL,
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_QUESTION,
                                   GTK_BUTTONS_YES_NO,
								   _("Do you want to save every refresh output?"),
                                   NULL);

    gtk_dialog_set_default_response (GTK_DIALOG (dialog),
                                     GTK_RESPONSE_ACCEPT);

	gtk_window_set_title (GTK_WINDOW (dialog), _("Auto save..."));
    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);

    g_signal_connect (G_OBJECT (dialog), "response",
                      G_CALLBACK (autorefresh_save_confirm_response),
                      NULL);

    gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
	
	return TRUE;
}

/* progress bar state */
void
progressbar_set (void)
{
	gchar *buffer;
	
	buffer = g_strdup_printf(_("Refresh number %d/%d"), ((guint)spin2 - cicle),(guint)spin2);
	
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (interface.progressbar), buffer);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (interface.progressbar), (spin2- (gdouble)cicle)/spin2);
	
	g_free (buffer);
}

/* widgets on/off */
void 
widgets_active (gboolean state)
{
	gtk_widget_set_sensitive (interface.spin1,state);
	gtk_widget_set_sensitive (interface.spin2,state);
	gtk_widget_set_sensitive (interface.progressbar,state);
	gtk_widget_set_sensitive (interface.stop,!state);
	gtk_widget_set_sensitive (interface.start,state);
	gtk_widget_set_sensitive (interface.autorefresh_button,state);
}

/* Main Signals Functions */
static gboolean
autorefresh_handler (gpointer data)
{
	on_refresh_clicked (NULL,NULL);
	
	if (state_save)
	{
		time_t t;
		struct tm *tm_ptr;
		gchar date_buffer[16];
	
		time (&t);
    	tm_ptr = localtime (&t);
	  	strftime (date_buffer, 16, "%Y%m%d%k%M%S", tm_ptr);
	
		g_free (filename);
		filename = g_strconcat (filename_copy,&date_buffer[0],NULL);
		
		save_output();
	}
	
	if (cicle == 0) return TRUE;
	
	cicle--;
	progressbar_set ();
	
	if (cicle == 0) {
		timeout_id = 0;
		
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (interface.progressbar), _("Finished"));
		
		if (state_save == TRUE)
			gtk_widget_set_sensitive (interface.save_as2, TRUE);

		widgets_active (TRUE);
		
		return FALSE;
	}
	
	return TRUE;
}

static void 
start_save_autorefresh (void)
{
	if (!strlen(filename))	return;
	
	timeout_id = gtk_timeout_add (time_step,autorefresh_handler,NULL);
}

void
start_autorefresh(GtkButton *button, gpointer user_data)
{
	
	spin1 = (gdouble) gtk_spin_button_get_value_as_int((GtkSpinButton *)interface.spin1);
	time_step = (guint) spin1 * 1000;
	spin2 = gtk_spin_button_get_value_as_int((GtkSpinButton *)interface.spin2);
	cicle = (guint) spin2;
	
	control_confirm();
	
	if (state_save == TRUE)
	{
		save_as_window();
		gtk_widget_set_sensitive (interface.save_as2, FALSE);
		
		g_free (filename_copy);
		filename_copy = g_strdup (filename);
	}
	
	widgets_active (FALSE);
	
	if (cicle == 0)
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (interface.progressbar), _("Endless"));
	else
		progressbar_set();
	
	timeout_id = gtk_timeout_add (time_step,autorefresh_handler,NULL);
}

void
stop_autorefresh(GtkButton *button, gpointer user_data)
{
	if (timeout_id) 
		{
			gtk_timeout_remove (timeout_id);	
			timeout_id = 0;
		}
		
	if (state_save == TRUE)
			gtk_widget_set_sensitive (interface.save_as2, TRUE);
		
	widgets_active (TRUE);
}
