/*  file selection - save output functions.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "autorefresh.h"
#include "file_selection.h"
#include "interface.h"


/* layouts */

/* layout 1 */
extern GtkListStore *command_info_table;

/* view_options */
extern gboolean state_fields[12];

/* interface */
extern main_window interface;

gchar *filename;

static gchar *format_fields[] = {
	"%-15s", "  %6s", "  %6s", "  %6s", "  %15s", "     %6s", " %4.4s", "  %10.10s", " %10.10s", " %6s ", " %10.10s", " %s" };

	
/* Main menu "save as" signal */
void
save_as_window (void)
{
	GtkWidget *dialog;
	
	dialog = gtk_file_chooser_dialog_new (_("Save output as..."),
				      GTK_WINDOW(interface.window),
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	{
		g_free (filename);
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	}
	
	gtk_widget_destroy (dialog);
}

void
on_save_as_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	save_as_window ();
	save_output();
}

/* start read and copy of infos in main table */

static gboolean
  foreach_func (GtkTreeModel *model,
                GtkTreePath  *path,
                GtkTreeIter  *iter,
                gpointer      user_data)
  {
    gchar *column[12];
	guint i;

	/* get single row */	  
    gtk_tree_model_get (model, iter,
                        0, &column[0], 1, &column[1], 2, &column[2], 3, &column[3],
	  					4, &column[4], 5, &column[5], 6, &column[6], 7, &column[7],
	  					8, &column[8], 9, &column[9], 10, &column[10], 11, &column[11],
	  					-1);

	for (i = 0; i < 12; i++)
	{
	      if (state_fields[i])
			  fprintf ((FILE *)user_data, format_fields[i], column [i]);
		  
		  g_free (column[i]);
	}
	
	fprintf ((FILE *)user_data, "\n");
	
    return FALSE; /* do not stop walking the store, call us with next row */
  }


void
save_output (void)
{
	FILE *file;
	guint i;
	
static const gchar *n_f[] = { "COMMAND", "PID", "PGID", "PPID", "USER", "FD", "TYPE", "DEVICE", "SIZE/OFF", "NLINK", "INODE", "NAME" };

    file = fopen ((const gchar *) filename, "w");
		  
    if (file == NULL)
		return;

    for (i = 0; i < 12; i++)
		if (state_fields[i])
			fprintf (file, format_fields[i], n_f[i]);
	
    fprintf (file, "\n");
	
	gtk_tree_model_foreach(GTK_TREE_MODEL(command_info_table), foreach_func, file);
		
	fclose (file);
	return;
}
