/*  interface - make main window and relative controls.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>

#include "file_selection.h"
#include "list_settings.h"
#include "interface.h"
#include "settings.h"
#include "preferences.h"
#include "autorefresh.h"
#include "about.h"
#include "refresh.h"
#include "layouts.h"


/* layouts */
extern GtkListStore *command_info_table;
extern GtkListStore *liststore_left_command_column;
extern GPtrArray *list_layout2_table;
extern GPtrArray *list_command_name;
extern GtkWidget *vbox_right_panel_side;
extern GtkWidget *view_left_command_column;
extern guint temp_l2elem_selected;

/* refresh */
extern gint num_command;
extern gint append_num;

/* list_int_addr */
extern GPtrArray *add_list;

/* main */
extern gchar *path_config;
extern GConfClient *client;
extern guint timeout_splash_window;

/* search */
extern gchar *name;
extern gchar *name_copy;

/* list_settings */
extern GPtrArray *item_array;

/* autorefresh */
extern gchar *filename;

/* extern */
main_window interface;
gchar *path;
gchar *commands_lsof;

static GtkWidget *splash_window = NULL;


/* autorefresh frame show & hide */

void
on_autorefresh_button_clicked (GtkButton * button, gpointer user_data)
{
	if (GTK_WIDGET_VISIBLE(interface.autorefresh_frame))
	{
		gtk_widget_hide (interface.autorefresh_frame);
		gtk_button_set_label (GTK_BUTTON(interface.autorefresh_button), "Show Autorefresh >>");
	}
	else
	{
		gtk_widget_show (interface.autorefresh_frame);
		gtk_button_set_label (GTK_BUTTON(interface.autorefresh_button), "<< Hide Autorefresh");
	}
}

/* signals interface */

void
on_refresh_clicked (GtkButton * button, gpointer user_data)
{
  gint i;
  gchar readbuf[BUFSIZ];

  /* sensitive FALSE for killpid-button */
  gtk_widget_set_sensitive (interface.killpid_button, FALSE);

  /* show wait-string on status bar */
  gnome_appbar_pop (GNOME_APPBAR(interface.statusbar));
  gnome_appbar_push (GNOME_APPBAR (interface.statusbar), _("please wait, glsof at work..."));

  /* clean lists */
  gtk_list_store_clear (command_info_table);
  gtk_list_store_clear (liststore_left_command_column);
	
  /* free list_command_name array */
  i = list_command_name->len - 1;
	
  if (i >= 0)
  {   
	  for (; i < 0; i--)
		  g_free (g_ptr_array_index(list_command_name, i));
		  
	  list_command_name = g_ptr_array_sized_new (0);
  }
  
  
  /* free list_layout2_table array */
  i = list_layout2_table->len - 1;
  
  if (i >= 0)
  {  
	  gtk_widget_destroy (vbox_right_panel_side);
	  vbox_right_panel_side = gtk_vbox_new (FALSE, 0);
	  gtk_widget_show (vbox_right_panel_side);
  	  gtk_paned_pack2 (GTK_PANED (interface.hpaned), vbox_right_panel_side, TRUE, TRUE);
	  
	  for (; i < 0; i--)
		  g_free (g_ptr_array_index(list_layout2_table, i));
	  
	  list_layout2_table = g_ptr_array_sized_new (0);
  }
  
  /* refresh */
  num_command = 0;
  update_clist ();
  
  /* command list names : focus first line*/
  if ( (list_command_name->len > 0) && (GTK_WIDGET_VISIBLE(interface.hpaned)) )
		gtk_widget_grab_focus (GTK_WIDGET (view_left_command_column));
  
  else if (!GTK_WIDGET_VISIBLE(interface.hpaned)) {
  /* show number of processes and lines in status bar */
  gnome_appbar_pop (GNOME_APPBAR(interface.statusbar));
  g_snprintf(readbuf, BUFSIZ, "Processes: %d\tTotal Lines: %d",list_command_name->len,append_num);
  gnome_appbar_push (GNOME_APPBAR (interface.statusbar), &readbuf[0]);
  }
}

void
on_window1_destroy (GtkObject * object, gpointer user_data)
{
  gint i;

  /* free memory */ 
  g_ptr_array_free (add_list,FALSE);
	
  for (i=list_command_name->len-1; i>=0; i--)
	  g_ptr_array_remove_index (list_command_name, i);
  	
  g_ptr_array_free (list_command_name,FALSE);
  
  g_ptr_array_free (item_array,FALSE);
	
  if (name_copy != NULL)
	  g_free (name_copy);
  if (name != NULL)
	  g_free (name);
  
  g_free (filename);
  
  save_size_and_position_main_window();
  
  gtk_main_quit ();
}

void 
on_item_layout_activate (GtkButton *button, gpointer user_data)
{
	gchar buf [256];

	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (interface.item_layout1)))
	{
		gtk_widget_show (interface.scrolledwindow);
		gtk_widget_hide (interface.hpaned);

		/* show number of processes and lines in status bar */
		gnome_appbar_pop (GNOME_APPBAR(interface.statusbar));
		g_snprintf(buf,256,"Processes: %d\tTotal Lines: %d",list_command_name->len,append_num);
		gnome_appbar_push (GNOME_APPBAR (interface.statusbar), &buf[0]);
		
		/* need error control */
		gconf_client_set_bool (client, "/apps/glsof/global/single_list",  TRUE, NULL);
	}
	else
	{
		guint len;

		gtk_widget_show (interface.hpaned);
		gtk_widget_hide (interface.scrolledwindow);
		
		len = list_command_name->len;
		
		if ( len > 0)
			{
				layout2_elem *elem;
				
				elem = g_ptr_array_index (list_layout2_table, temp_l2elem_selected);
				
				/* show line numbers in status bar */
				gnome_appbar_pop (GNOME_APPBAR(interface.statusbar));
				g_snprintf(buf, 256, "Lines: %d\tTotal Lines: %d", elem->lines,append_num);
				gnome_appbar_push (GNOME_APPBAR (interface.statusbar), &buf[0]);
				
				/* set focus */
				gtk_widget_grab_focus (GTK_WIDGET (view_left_command_column));
			}
			
			/* need error control */
			gconf_client_set_bool (client, "/apps/glsof/global/single_list",  FALSE, NULL);
		}
}

void 
on_view_statusbar_menu_toggled (GtkCheckMenuItem *checkmenuitem,
                                            gpointer user_data)
{
	gboolean state = gtk_check_menu_item_get_active (checkmenuitem);
	
	if (state)
		gtk_widget_show (interface.statusbar);
	else
		gtk_widget_hide (interface.statusbar);
	
	gconf_client_set_bool (client, "/apps/glsof/global/show_statusbar",  state, NULL);
}


/************************************/
/*      	MAKE SPLASH WINDOW      */
/************************************/

void 
make_splash_window(void)
{
	GtkWidget *vbox;
	GdkPixbuf* pixbuf;
	GtkWidget *image;
	GtkWidget *label;
	GdkColor color;
	
	splash_window = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_window_set_position (GTK_WINDOW (splash_window), GTK_WIN_POS_CENTER);
    gtk_window_set_decorated(GTK_WINDOW(splash_window), FALSE);
    gtk_window_set_resizable(GTK_WINDOW(splash_window),FALSE);
	color.red = 65535;
    color.blue = 65535;
    color.green = 65535;
    gtk_widget_modify_bg(splash_window, GTK_STATE_NORMAL,&color);

	vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(splash_window), vbox);
	gtk_widget_show(vbox);

	gtk_widget_realize(splash_window);
	gdk_window_set_decorations(splash_window->window, 0);
	
	pixbuf = gdk_pixbuf_new_from_file(PIXMAPS_PATH "glsof-logo.png", NULL);
	image = gtk_image_new_from_pixbuf(pixbuf);
	gtk_container_add (GTK_CONTAINER (vbox), image);
	g_object_unref(pixbuf);	
	gtk_widget_show(image);
	
	label = gtk_label_new(_("please wait, glsof at work..."));
	gtk_container_add (GTK_CONTAINER (vbox), label);
    gtk_widget_show(label);
		
	gtk_widget_show(splash_window);
	
	while(g_main_context_pending(NULL) || !GTK_WIDGET_REALIZED(image))
		gtk_main_iteration();
}

/************************************/
/*      CLOSE SPLASH WINDOW         */
/************************************/

gboolean 
close_splash_window (gpointer data)
{
	gtk_timeout_remove (timeout_splash_window);
	gtk_widget_destroy(splash_window);
	splash_window = NULL;
		
	return TRUE;
}


/* Main Window */

void
make_main_window (void)
{
  GladeXML *xml_glade;
  GtkWidget *vbox;
  GtkWidget *item_layout2;
  GtkWidget *view_statusbar_menu;
  GtkAccelGroup *accel_group;
	
  
  accel_group = gtk_accel_group_new ();
  
  xml_glade = glade_xml_new (GLADE_PATH"glsof_mainwindow.glade", NULL, NULL);
  interface.window = glade_xml_get_widget (xml_glade, "glsof-mainwindow");
  gtk_window_add_accel_group (GTK_WINDOW (interface.window), accel_group);
  vbox = glade_xml_get_widget (xml_glade, "vbox");
  interface.handlebox = glade_xml_get_widget (xml_glade, "bonobodockitem2");
  interface.statusbar = glade_xml_get_widget (xml_glade, "appbar1");
  view_statusbar_menu = glade_xml_get_widget (xml_glade, "menu_statusbar1");
  interface.item_layout1 = glade_xml_get_widget (xml_glade, "singlelist1");
  item_layout2 = glade_xml_get_widget (xml_glade, "doublelist1");
  interface.save_as2 = glade_xml_get_widget (xml_glade, "save_output1");
  interface.scrolledwindow = glade_xml_get_widget (xml_glade, "scrolledwindow1");
  interface.hpaned = glade_xml_get_widget (xml_glade, "hpaned1");
  interface.autorefresh_frame = glade_xml_get_widget (xml_glade, "autorefresh_frame");
  interface.change_settings_sub_menu = glade_xml_get_widget (xml_glade, "change_settings");
  interface.killpid_button = glade_xml_get_widget (xml_glade, "killpid_button");
  
  /* autorefresh widgets */
  interface.spin1 = glade_xml_get_widget (xml_glade, "spinbutton1");
  interface.spin2 = glade_xml_get_widget (xml_glade, "spinbutton2");
  interface.progressbar = glade_xml_get_widget (xml_glade, "progressbar1");
  interface.start = glade_xml_get_widget (xml_glade, "start_button");
  interface.stop = glade_xml_get_widget (xml_glade, "stop_button");
  interface.autorefresh_button = glade_xml_get_widget (xml_glade, "autorefresh_button");
  
  /* search */
  interface.search_button = glade_xml_get_widget (xml_glade,"search_button");
  interface.search_entry = glade_xml_get_widget (xml_glade,"search_entry");
  interface.combox = glade_xml_get_widget (xml_glade,"combobox");
  gtk_combo_box_set_active (GTK_COMBO_BOX (interface.combox), 0);
  
  get_size_and_position_main_window ();
	
  glade_xml_signal_autoconnect(xml_glade);
  
  /* show & hide layouts */
  make_layout_1 (vbox);
  make_layout_2 (vbox);
  
  /* hide autorefresh_frame */
  gtk_widget_hide (interface.autorefresh_frame);
	
  /* make array of pointers for 'list addresses' interface */
  add_list = g_ptr_array_new ();
  list_layout2_table = g_ptr_array_new ();
  list_command_name = g_ptr_array_new ();
  
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (interface.item_layout1),
gconf_client_get_bool (client, "/apps/glsof/global/single_list", NULL));
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (view_statusbar_menu),
gconf_client_get_bool (client, "/apps/glsof/global/show_statusbar", NULL));
  on_item_layout_activate (NULL,NULL);
}
