/*  layouts - make single and double layers.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "layouts.h"
#include "interface.h"
#include "popupmenu.h"

/* refresh */
extern GPtrArray *list_layout2_table;
extern guint append_num;

/* interface */
extern main_window interface;

/* extern */

/* layout 1 */
GtkWidget *view;
GtkListStore *command_info_table;
GtkTreeSelection *selection_single_list;

/* layout 2 */
GtkWidget *vbox_right_panel_side;
GtkWidget *view_left_command_column;
GtkListStore *liststore_left_command_column;
GtkTreeSelection *selection_left_command_column;
  
guint temp_l2elem_selected;

gchar *name_fields[] = {
  "COMMAND       ", "PID    ", "PGID    ", "PPID  ", "USER      ", "FD   ",
  "TYPE      ", "DEVICE    ", "SIZE     ", "NLINK   ", "NODE  ", "NAME"
};
/* _extern */

/********************************/
/*           LAYOUT 1           */
/********************************/

void 
make_layout_1 (GtkWidget *vbox1)
{
  GtkCellRenderer *cell;
  GtkTreeViewColumn *column;
  int i = 0;


  /* make main column-list */
  view = gtk_tree_view_new ();
  selection_single_list = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));

  gtk_tree_selection_set_mode (selection_single_list, GTK_SELECTION_SINGLE);
  command_info_table = gtk_list_store_new (12, \
  										G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, \
										G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, \
  										G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, \
										G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  /* white - grey alternate */
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
  
  gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (command_info_table));
				 
  g_object_unref(G_OBJECT(command_info_table));

  
  for (i = 0; i < 12; i++)
    {
		cell = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes(name_fields[i],cell,"text",i,NULL);
		gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
		
		if ( (i==0) || (i==11) )
			gtk_tree_view_column_set_sort_column_id(column, i);
		
		gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
	}
	
  /* popup menu signals */
  g_signal_connect (G_OBJECT (selection_single_list), "changed", G_CALLBACK (catch_pid), NULL);
  g_signal_connect(view, "button-press-event", (GCallback) view_onButtonPressed, NULL);
	
  gtk_widget_show (view);
  gtk_container_add (GTK_CONTAINER (interface.scrolledwindow), view);
}


/********************************/
/*           LAYOUT 2           */
/********************************/

/* get row from menu */
static void
selection_changed_layout_table (GtkTreeSelection * selection, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		GtkTreePath *path;
		layout2_elem *elem;
		
		gchar buf[256];
		
		path = gtk_tree_model_get_path (model, &iter);
		
		elem = g_ptr_array_index (list_layout2_table, temp_l2elem_selected);
		gtk_widget_hide (elem->scrolledwindow);
		
		temp_l2elem_selected = gtk_tree_path_get_indices (path)[0];
		elem = g_ptr_array_index (list_layout2_table, temp_l2elem_selected);
		gtk_widget_show (elem->scrolledwindow);
		
		gtk_tree_path_free (path);
		
		/* show line numbers in status bar */
		
		gnome_appbar_pop (GNOME_APPBAR(interface.statusbar));
		g_snprintf(buf, 256, "Lines: %d\tTotal Lines: %d", elem->lines,append_num);
		gnome_appbar_push (GNOME_APPBAR (interface.statusbar), &buf[0]);
	}
}				/* _selection_changed */


void
make_layout_2 (GtkWidget *vbox1)
{
  GtkCellRenderer *cell_left_command_column;
  GtkWidget *scrolledwindow1;
  GtkTreeViewColumn *column;

  
  /* add left COMMAND's columm */
  
  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_paned_pack1 (GTK_PANED (interface.hpaned), scrolledwindow1, FALSE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
				GTK_POLICY_AUTOMATIC,
				GTK_POLICY_AUTOMATIC);
  
  view_left_command_column = gtk_tree_view_new ();
  selection_left_command_column = gtk_tree_view_get_selection (GTK_TREE_VIEW (view_left_command_column));

  gtk_tree_selection_set_mode (selection_left_command_column, GTK_SELECTION_SINGLE);
  liststore_left_command_column = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);

  gtk_tree_view_set_model (GTK_TREE_VIEW (view_left_command_column),
			 GTK_TREE_MODEL (liststore_left_command_column));
			 
  g_object_unref(G_OBJECT(liststore_left_command_column));

  cell_left_command_column = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("  #  ", cell_left_command_column, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(view_left_command_column), column);
						 
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  cell_left_command_column = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("COMMAND", cell_left_command_column, "text", 1, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(view_left_command_column), column);
						 
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  gtk_widget_show (view_left_command_column);
	
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), view_left_command_column);
  
  /* second panel side */
  
  vbox_right_panel_side = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox_right_panel_side);
  gtk_paned_pack2 (GTK_PANED (interface.hpaned), vbox_right_panel_side, TRUE, TRUE);
  
  /* signals */
  
  g_signal_connect (G_OBJECT (selection_left_command_column), "changed", 
  			  G_CALLBACK (selection_changed_layout_table), NULL);
}
