/*  list int addr - Make window for internet addresses/services/ports control.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <string.h>

#include "list_int_addr.h"

#define MAX_LIST_ELEMENTS	100	/* lsof controls max 100 (-i) options */

/* list items */
enum
{
	NUMBER_ITEM = 0,
	ITEM,
	NUM_COLS
};

/* main */
extern GConfClient *client;

/* extern */
GPtrArray *add_list;
list_add lia;


/* Signal Functions */
static void
selection_changed (GtkTreeSelection * selection, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		GtkTreePath *path;
		gint position;
		node_data *tmp_element;

		path = gtk_tree_model_get_path (model, &iter);
		position = gtk_tree_path_get_indices (path)[0];
		tmp_element = g_ptr_array_index (add_list, position);
		gtk_tree_path_free (path);

		gtk_entry_set_text (GTK_ENTRY (lia.host_entry), tmp_element->host);
		gtk_entry_set_text (GTK_ENTRY (lia.port_entry), tmp_element->port);
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (lia.ipv_combo)->entry), tmp_element->ipv);
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (lia.prot_combo)->entry), tmp_element->protocol);
		
		gtk_widget_set_sensitive (lia.edit_button, TRUE);
		gtk_widget_set_sensitive (lia.remove_button,TRUE);
	}
}				/* _selection_changed */

void
on_close_internet_addresses_window_clicked (GtkButton * button, gpointer user_data)
{
	gtk_widget_hide (lia.window);
}

/* Elements List Controls */
void
add_list_element (GtkButton * button, gpointer data)
{
	GtkTreeIter iter;

	if (add_list->len < MAX_LIST_ELEMENTS)
	{
		int len = 0;

		/* Create new struct on heap - zero's memory */
		node_data *tmp_node = g_new0 (node_data, 1);
		
		tmp_node->ipv = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (lia.ipv_combo)->entry)));
        tmp_node->protocol = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (lia.prot_combo)->entry)));

		tmp_node->option = g_strconcat (tmp_node->ipv, tmp_node->protocol, NULL);

		tmp_node->host = g_strdup (gtk_entry_get_text (GTK_ENTRY (lia.host_entry)));
		if (strlen (tmp_node->host))
			tmp_node->option = g_strconcat (tmp_node->option, "@", tmp_node->host, NULL);

		tmp_node->port = g_strdup (gtk_entry_get_text (GTK_ENTRY (lia.port_entry)));
		if (strlen (tmp_node->port))
			tmp_node->option = g_strconcat (tmp_node->option, ":", tmp_node->port, NULL);

		len = strlen (tmp_node->option);

		if (len > 0)
		{
			int i;
			gboolean state = FALSE;

			for (i = 0; i < add_list->len; i++)
			{
				node_data *elem = g_ptr_array_index (add_list, i);

				if (!g_ascii_strcasecmp (tmp_node->option, elem->option))
				{
					GtkWidget *dialog;
					
					state = TRUE;
					g_free (tmp_node);
		
					dialog = gtk_message_dialog_new (GTK_WINDOW(lia.window),
        			                           GTK_DIALOG_DESTROY_WITH_PARENT,
            			                       GTK_MESSAGE_ERROR,
                			                   GTK_BUTTONS_OK,
                    			               _("You already have \"%s\" to check."),
                        			           elem->option);
		
					gtk_dialog_run (GTK_DIALOG (dialog));
 					gtk_widget_destroy (dialog);
					
					break;
				}
			}

			if (state == FALSE)
			{
				GtkTreeIter iter;
				
				/* add new pointer in add_list */
				g_ptr_array_add (add_list, (gpointer) tmp_node);

				gtk_entry_set_text (GTK_ENTRY (lia.host_entry), "");
				gtk_entry_set_text (GTK_ENTRY (lia.port_entry), "");
				gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (lia.ipv_combo)->entry), "");
				gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (lia.prot_combo)->entry), "");
				
				gtk_list_store_append (lia.liststore_right, &iter);
				gtk_list_store_set (lia.liststore_right, &iter, NUMBER_ITEM, add_list->len,ITEM, tmp_node->option,-1);
				
				gtk_widget_set_sensitive (lia.edit_button,FALSE);
				
				return;
			}
		}
		else
		{
			GtkWidget *dialog;
		
			dialog = gtk_message_dialog_new (GTK_WINDOW(lia.window),
        		                           GTK_DIALOG_DESTROY_WITH_PARENT,
            		                       GTK_MESSAGE_ERROR,
                		                   GTK_BUTTONS_OK,
                    		               "%s",
                        		           _("You must fill at least one field."));
		
			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
			
			g_free (tmp_node);
		}

	}

}				/* _add_list_element */


void
modify_list_element (GtkButton * button, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected (GTK_TREE_SELECTION (lia.selection), &model, &iter))
	{
		GtkTreePath *path;
		gint position = 0;
		gint i;
		node_data *tmp_element;

		/* temp datas */
		gchar *option;
		gchar *ipv;
		gchar *protocol;
		gchar *host;
		gchar *port;

		ipv = g_strdup (gtk_entry_get_text
				(GTK_ENTRY (GTK_COMBO (lia.ipv_combo)->entry)));
		protocol =
			g_strdup (gtk_entry_get_text
				  (GTK_ENTRY (GTK_COMBO (lia.prot_combo)->entry)));
		option = g_strconcat (ipv, protocol, NULL);

		host = g_strdup (gtk_entry_get_text (GTK_ENTRY (lia.host_entry)));
		if (strlen (host))
			option = g_strconcat (option, "@", host, NULL);

		port = g_strdup (gtk_entry_get_text (GTK_ENTRY (lia.port_entry)));
		if (strlen (port))
			option = g_strconcat (option, ":", port, NULL);

		if (strlen (option) == 0)
			{
				GtkWidget *dialog;
		
				dialog = gtk_message_dialog_new (GTK_WINDOW(lia.window),
        		                       GTK_DIALOG_DESTROY_WITH_PARENT,
            	                       GTK_MESSAGE_ERROR,
                	                   GTK_BUTTONS_OK,
                   		               "%s",
                       		           _("You must fill at least one field."));
				
				gtk_dialog_run (GTK_DIALOG (dialog));
				gtk_widget_destroy (dialog);
				
				return;
			}

		for (i = 0; i < add_list->len; i++)
		{
			tmp_element = g_ptr_array_index (add_list, i);

			if (!g_ascii_strcasecmp (tmp_element->option, option))
			{
				GtkWidget *dialog;
					
				dialog = gtk_message_dialog_new (GTK_WINDOW(lia.window),
        		                           GTK_DIALOG_DESTROY_WITH_PARENT,
            		                       GTK_MESSAGE_ERROR,
                		                   GTK_BUTTONS_OK,
                   			               _("You already have \"%s\" to check."),
                       			           tmp_element->option);
		
				gtk_dialog_run (GTK_DIALOG (dialog));
 				gtk_widget_destroy (dialog);
				
				return;
			}
		}

		path = gtk_tree_model_get_path (model, &iter);
		position = gtk_tree_path_get_indices (path)[0];
		tmp_element = g_ptr_array_index (add_list, position);
		gtk_tree_path_free (path);

		tmp_element->ipv = g_strdup (ipv);
		tmp_element->protocol = g_strdup (protocol);
		tmp_element->host = g_strdup (host);
		tmp_element->port = g_strdup (port);
		tmp_element->option = g_strdup (option);

		gtk_list_store_set (lia.liststore_right, &iter, 1, tmp_element->option, -1);
		
		g_free (option);
		g_free (ipv);
		g_free (protocol);
		g_free (host);
		g_free (port);
		
		gtk_widget_set_sensitive (lia.edit_button, FALSE);
	}

}				/* _modify_list_element */


void
remove_list_element (GtkButton * button, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;


	if (gtk_tree_selection_get_selected (GTK_TREE_SELECTION (lia.selection), &model, &iter))
	{
		GtkTreePath *path;
		guint position;
		
		path = gtk_tree_model_get_path (model, &iter);

		position = gtk_tree_path_get_indices (path)[0];
		g_ptr_array_remove_index (add_list, position);
		
		/* remove line from table */
		
		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
		gtk_tree_path_free (path);
		
		while (position < add_list->len)
		{
			position++;
			gtk_list_store_set (lia.liststore_right, &iter, 0, position, -1);
			gtk_tree_model_iter_next (model, &iter);
		}

		gtk_entry_set_text (GTK_ENTRY (lia.host_entry), "");
		gtk_entry_set_text (GTK_ENTRY (lia.port_entry), "");
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (lia.ipv_combo)->entry), "");
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (lia.prot_combo)->entry), "");
		
		gtk_widget_set_sensitive (lia.remove_button,FALSE);
		gtk_widget_set_sensitive (lia.edit_button,FALSE);
	}
}				/* _remove_list_element */

/* make set list addresses */
void
on_set_list_clicked (GtkButton * button, gpointer user_data)
{
	gtk_widget_show (lia.window);
}

void
make_internet_addresses_window (void)
{
	GladeXML *xml_glade;
	GtkTreeViewColumn *column;
  	GtkCellRenderer *cell;
	GtkTreeIter iter;
	GtkWidget *list_treeview;
	GtkWidget *edit_button;
	GtkWidget *add_button;
	GtkWidget *remove_button;
	GtkWidget *close_button;
	GtkWidget *table;
	GtkWidget *scrolledwindow;
	GtkWidget *ipv_combo_entry;
	GtkWidget *prot_combo_entry;
	GList *combo1_items = NULL;
	GList *combo2_items = NULL;
	
	
	xml_glade = glade_xml_new (GLADE_PATH"glsof_list_int_addr.glade", NULL, NULL);
  	lia.window = glade_xml_get_widget (xml_glade, "internet-window");
	lia.host_entry = glade_xml_get_widget (xml_glade, "host_entry");
	lia.port_entry = glade_xml_get_widget (xml_glade, "port_entry");
	
	table = glade_xml_get_widget (xml_glade, "table_entry");
	add_button = glade_xml_get_widget (xml_glade, "add_button");
	close_button = glade_xml_get_widget (xml_glade, "close_button");
	scrolledwindow = glade_xml_get_widget (xml_glade, "scrolledwindow");
	
	lia.edit_button = glade_xml_get_widget (xml_glade, "edit_button");
	lia.remove_button = glade_xml_get_widget (xml_glade, "remove_button");
	
	/* list for options */
	lia.view = gtk_tree_view_new ();
    lia.selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (lia.view));

    gtk_tree_selection_set_mode (lia.selection, GTK_SELECTION_SINGLE);
    lia.liststore_right = gtk_list_store_new (NUM_COLS, G_TYPE_INT, G_TYPE_STRING);

    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (lia.view), TRUE); /* white - grey alternate */

    gtk_tree_view_set_model (GTK_TREE_VIEW (lia.view), GTK_TREE_MODEL (lia.liststore_right));

    cell = gtk_cell_renderer_text_new ();
    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (lia.view), -1,"   #  ", cell, "text",NUMBER_ITEM, NULL);

    cell = gtk_cell_renderer_text_new ();
    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (lia.view), -1,"Addresses|ports|services", cell, "text",ITEM, NULL);

    gtk_widget_show (GTK_WIDGET(lia.view));

    gtk_container_add (GTK_CONTAINER (scrolledwindow), GTK_WIDGET(lia.view));
	
	/* comboboxes */
	lia.ipv_combo = gtk_combo_new ();
    gtk_widget_show (lia.ipv_combo);
    combo1_items = g_list_append (combo1_items, (gpointer) "");
    combo1_items = g_list_append (combo1_items, (gpointer) ("4"));
    combo1_items = g_list_append (combo1_items, (gpointer) ("6"));
    gtk_combo_set_popdown_strings (GTK_COMBO (lia.ipv_combo), combo1_items);
    gtk_combo_set_value_in_list (GTK_COMBO (lia.ipv_combo),TRUE,TRUE);
    g_list_free (combo1_items);

    ipv_combo_entry = GTK_COMBO (lia.ipv_combo)->entry;
    gtk_widget_show (ipv_combo_entry);

    lia.prot_combo = gtk_combo_new ();
    gtk_widget_show (lia.prot_combo);
    combo2_items = g_list_append (combo2_items, (gpointer) "");
    combo2_items = g_list_append (combo2_items, (gpointer) _("TCP"));
    combo2_items = g_list_append (combo2_items, (gpointer) _("UDP"));
    gtk_combo_set_popdown_strings (GTK_COMBO (lia.prot_combo), combo2_items);
    gtk_combo_set_value_in_list (GTK_COMBO (lia.prot_combo),TRUE,TRUE);
    g_list_free (combo2_items);

    prot_combo_entry = GTK_COMBO (lia.prot_combo)->entry;
    gtk_widget_show (prot_combo_entry);

	/* put in the table comboboxes */
	gtk_table_attach (GTK_TABLE(table), lia.ipv_combo, 1,2, 2,3, GTK_FILL,GTK_FILL, 0,0);
	gtk_table_attach (GTK_TABLE(table), lia.prot_combo, 1,2, 3,4, GTK_FILL,GTK_FILL, 0,0);
	gtk_widget_show (lia.ipv_combo);
	gtk_widget_show (lia.prot_combo);
	
	/* signals */
	g_signal_connect (G_OBJECT (lia.selection), "changed", G_CALLBACK (selection_changed),NULL);
	
	glade_xml_signal_autoconnect(xml_glade);
    g_object_unref(G_OBJECT(xml_glade));
}
