/*  list settings - Edit, create and remove 'settings'.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

#include "settings.h"
#include "list_settings.h"
#include "interface.h"

#define GCONF_LIST_SETTINGS_PATH	"/apps/glsof/global/settings_list"
#define GCONF_LIST_SETTINGS_STRING "/apps/glsof/global/launch_settings"

/* main */
extern GConfClient *client;

/* interface */
extern main_window interface;	/* main window */

/* settings */
extern settings_window settings;

/* extern */
list_settings_window window;
GPtrArray *item_array;

static new_settings_dialog dialog;
static gboolean status_dialog;


static void
refresh_for_settings_selected (gchar *settings_name)
{
	if (settings.gconf_path_settings != NULL)
		g_free (settings.gconf_path_settings);
	if (settings.save_new_settings_name != NULL)
		g_free (settings.save_new_settings_name);
	
	settings.save_new_settings_name = g_strdup (settings_name);
	settings.gconf_path_settings = g_strconcat("/apps/glsof/settings/", settings.save_new_settings_name, NULL);

	load_sets();
	on_refresh_clicked (NULL, NULL);
}

static void 
menuitem_settings_response (GtkCheckMenuItem *checkmenuitem, gpointer settings_name)
{
	if (!gtk_check_menu_item_get_active (checkmenuitem))
		return;
	
	refresh_for_settings_selected ((gchar *)settings_name);
}


static void
add_item_radio (gpointer data)
{
	GtkWidget *item;
	static GSList *group_radio;
	
	/* radio settings sub menu */
	item = gtk_radio_menu_item_new_with_label (group_radio, data);
	
	g_ptr_array_add (item_array, (gpointer) item);
	
  	group_radio = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
	
	gtk_menu_shell_append (GTK_MENU_SHELL (interface.settings_radio_menu), item);
	
	/* Launch 'settings' when the menuitem is selected */
	g_signal_connect (G_OBJECT (item), "toggled",	
		                     G_CALLBACK (menuitem_settings_response), 
                             (gpointer) g_strdup (data));
	
	gtk_widget_show (item);
}


static void
handler_list (gpointer data, gpointer user_data)
{
	GtkTreeIter iter;
	
	gtk_list_store_append (GTK_LIST_STORE(window.liststore), &iter);
	gtk_list_store_set (window.liststore, &iter, 0, data, -1);
	
	/* make radio menu 'settings' */
	add_item_radio(data);
}

static void
load_list (void)
{
	window.list = gconf_client_get_list (client, GCONF_LIST_SETTINGS_PATH, GCONF_VALUE_STRING, NULL);
	
	/* item menu array */
	item_array = g_ptr_array_new ();
	
	interface.settings_radio_menu = gtk_menu_new ();
	
	g_slist_foreach (window.list, (GFunc) handler_list, NULL);
	
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface.change_settings_sub_menu), interface.settings_radio_menu);
}

/***************************/
/* Edit, Delete and Create */
/***************************/
/* create */
void
on_create_list_settings_clicked (GtkButton *button, gpointer user_data)
{
	gtk_widget_show (dialog.dialog);
}

/* edit */
void
on_edit_list_settings_clicked (GtkButton *button, gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;

	window.selection_list = gtk_tree_view_get_selection(GTK_TREE_VIEW(window.view_list));
	if (gtk_tree_selection_get_selected (window.selection_list, &model, &iter))
	{
		g_free (settings.gconf_path_settings);
		g_free (settings.save_new_settings_name);
		
		gtk_tree_model_get (model, &iter, 0, &settings.save_new_settings_name, -1);
		settings.gconf_path_settings = g_strconcat("/apps/glsof/settings/", settings.save_new_settings_name, NULL);
		
		load_sets();
		
		change_title_settings_window();
		
		gtk_widget_show (settings.window);
	}
}

/* remove */
static void
remove_dir (gchar *path)
{
  /* general - id or login */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "id"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "login"), NULL );
	  
  /* general - offset or size */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "offset"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "size"), NULL );

  /* general - all AND */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "and"), NULL );
  
  /* general - hide 'lsof' and 'glsof' from output*/
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "hideglsof"), NULL );
	  
  /* files - processes name exec */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "procexe"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "procexe_entry"), NULL );
  
  /* files - processes ID */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "procid"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "procid_entry"), NULL );
  
  /* files - processes PGID */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "procpgid"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "procpgid_entry"), NULL );
  
  /* processes - display directory */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "disdir"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "disdir_entry"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "depth"), NULL );
  
  /* processes - files descriptors */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "fdlistdesc"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "fdlistdesc_entry"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "incexc"), NULL );
  
  /* processes - list of files for id or username */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "listidname"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "listidname_entry"), NULL );
  
  /* processes - max number of links for a file */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "numberlinks"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "numberlinks_entry"), NULL );
  
  /* network - list of files for internet addresses */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "internetadds"), NULL );
  /* miss list n = tmp_add_list */
  
  /* network - display ip-form addresses */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "ipadds"), NULL );
  
  /* network - port number */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "ports"), NULL );
  
  /* network - nfs */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "nfs"), NULL );
  
  /* network - unix sockets */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "unixsockets"), NULL );
  
  /* kernel - timeout seconds */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "timeoutsecs"), NULL );
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "timeoutsecs_entry"), NULL );
  
  /* kernel - avoid */
  gconf_client_unset (client, gconf_concat_dir_and_key (path, "avoid"), NULL );
}

static void
delete_confirm_response (GtkWidget   *dialog, int response_id, GtkWindow   *transient_parent)
{
	if (response_id == GTK_RESPONSE_ACCEPT)
		status_dialog = TRUE;
}

static gboolean 
control_confirm (gchar *tmp_name)
{
	GtkWidget *dialog;
		
	status_dialog = FALSE;

	/* delete: YES or NO */
	dialog = gtk_message_dialog_new (GTK_WINDOW(window.window),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_QUESTION,
                                   GTK_BUTTONS_NONE,
								   _("Delete settings \"%s\"?"),
                                   tmp_name);

    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                            GTK_STOCK_CANCEL,
                            GTK_RESPONSE_REJECT,
                            GTK_STOCK_DELETE,
                            GTK_RESPONSE_ACCEPT,
                            NULL);

    gtk_dialog_set_default_response (GTK_DIALOG (dialog),
                                     GTK_RESPONSE_ACCEPT);

	gtk_window_set_title (GTK_WINDOW (dialog), _("Delete Settings"));
    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);

    g_signal_connect (G_OBJECT (dialog), "response",
                      G_CALLBACK (delete_confirm_response),
                      NULL);

    gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
	
	return status_dialog;
}


void
on_remove_list_settings_clicked (GtkButton *button, gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	
	/* at least one 'settings' */
	if (g_slist_length(window.list)==1)
	{
		GtkWidget *dialog;
		
		dialog = gtk_message_dialog_new (GTK_WINDOW(window.window),
        	                           GTK_DIALOG_DESTROY_WITH_PARENT,
            	                       GTK_MESSAGE_ERROR,
                	                   GTK_BUTTONS_OK,
                    	               "%s",
                        	           _("You must have at least one 'settings'; you can't delete all of them."));
		
		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
		
		return;
	}

    /* Confirm delete settings */
	if (gtk_tree_selection_get_selected (window.selection_list, &model, &iter))
	{
		gchar *tmp_name;
		GtkTreePath *path;
		GtkWidget *item_radio_menu;
		gint index_list;
		gint index_active;
		gboolean save_state;
		
		gtk_tree_model_get (model, &iter, 0, &tmp_name, -1);
		if (control_confirm (tmp_name) == FALSE)
		{
			g_object_unref(G_OBJECT(model));
			g_free (tmp_name);
			return;
		}
		
		path = gtk_tree_model_get_path (model, &iter);
		
		index_list = gtk_tree_path_get_indices (path)[0];
		index_active = gtk_combo_box_get_active (GTK_COMBO_BOX(window.launch_combobox));

		/* remove name in all places */
		g_slist_remove (window.list, g_slist_nth_data(window.list, index_list));
		gtk_list_store_remove(GTK_LIST_STORE (window.liststore), &iter);
		
		/* if remove same 'settings' for launch at start, set combobox at first name in list */
		if (index_list == index_active)
			gtk_combo_box_set_active (GTK_COMBO_BOX(window.launch_combobox),0);
		
		/* remove radio menu item */
		item_radio_menu = g_ptr_array_index (item_array, index_list);
		/*
		** if remove same 'settings' selected in 'change settings', 
		** set 'change settings' at first name in list
		*/
		save_state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(item_radio_menu));
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(g_ptr_array_index (item_array, 0)),FALSE);
		
		gtk_container_remove (GTK_CONTAINER(interface.settings_radio_menu), item_radio_menu);
		g_ptr_array_remove_index (item_array, index_list);
		
		if (save_state == TRUE)
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (g_ptr_array_index (item_array, 0)),TRUE);
		
		/* remove 'settings dir' */
		remove_dir (g_strconcat("/apps/glsof/settings/", tmp_name, NULL));
		
		gconf_client_set_list (client, GCONF_LIST_SETTINGS_PATH, GCONF_VALUE_STRING, window.list, NULL);

		g_object_unref(G_OBJECT(model));
		gtk_tree_path_free (path);
		g_free (tmp_name);
	}
}


/************************/
/* List Settings Window */
/************************/
void
on_list_settings_clicked (GtkButton * button, gpointer user_data)
{
	/* signal from menu */
	gtk_widget_show (window.window);
}

void
on_close_list_settings_clicked (GtkButton * button, gpointer user_data)
{
	gtk_widget_hide (window.window);
}

/* selection signals - 2 little functions stolen from gnome-terminal 2.7.3 :)*/
static void
count_selected_settings_func (GtkTreeModel      *model,
                              GtkTreePath       *path,
                              GtkTreeIter       *iter,
                              gpointer           data)
{
  int *count = data;

  *count += 1;
}

static void
on_changed_list_settings_selection (GtkTreeSelection *treeselection, gpointer user_data)
{
	int count;

  	count = 0;
  	gtk_tree_selection_selected_foreach (window.selection,
    	                                   count_selected_settings_func,
        	                               &count);

  	gtk_widget_set_sensitive (window.edit_button, count == 1);
  	gtk_widget_set_sensitive (window.remove_button, count > 0);
}

static void
on_changed_list_settings_combobox (GtkComboBox *combobox, gpointer user_data)
{
	gboolean valid;
	GtkTreeIter iter;
	
	valid = gtk_combo_box_get_active_iter (GTK_COMBO_BOX(window.launch_combobox), &iter);
	
	if (valid == TRUE)
	{
		gchar *settings_selected;
		
		gtk_tree_model_get (GTK_TREE_MODEL(window.liststore), &iter, 0, &settings_selected, -1);
		gconf_client_set_string (client, GCONF_LIST_SETTINGS_STRING, settings_selected,NULL);
		
		g_free (settings_selected);
	}
}

void
make_list_settings_window (void)
{
  GladeXML *xml_glade;
  GtkTooltips *button_bar_tips;
  GtkTreeViewColumn *column;
  GtkCellRenderer *cell;
  GtkWidget *scrolledwindow;
  GtkWidget *hbox;
  GtkTreeIter iter;
	
  gchar *next_string;
  gchar *match_string;
  gboolean valid;
  gboolean exit = FALSE;
  guint index_item = 0;
	

  button_bar_tips = gtk_tooltips_new ();
	
  xml_glade = glade_xml_new (GLADE_PATH"glsof_list_settings.glade", NULL, NULL);
	
  /* window */
  window.window = glade_xml_get_widget (xml_glade, "glsof-list_of_settings");

  hbox= glade_xml_get_widget (xml_glade, "hbox3");
  scrolledwindow = glade_xml_get_widget (xml_glade, "scrolledwindow1");
  
  window.edit_button = glade_xml_get_widget (xml_glade, "edit_button");
  window.remove_button = glade_xml_get_widget (xml_glade, "remove_button");
	
  /* tree-model for list */
  window.view_list = gtk_tree_view_new ();
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(window.view_list), FALSE);
  window.selection_list = gtk_tree_view_get_selection (GTK_TREE_VIEW (window.view_list));
  gtk_tree_selection_set_mode (window.selection_list, GTK_SELECTION_SINGLE);
  window.liststore = gtk_list_store_new (1, G_TYPE_STRING);
  gtk_tree_view_set_model (GTK_TREE_VIEW (window.view_list), GTK_TREE_MODEL (window.liststore));
  cell = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("", cell, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(window.view_list), column);
  gtk_widget_show (window.view_list);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), window.view_list);
  
  /* tree-model for combobox */
  window.launch_combobox = gtk_combo_box_new_with_model (GTK_TREE_MODEL(window.liststore));
  cell = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (window.launch_combobox), cell, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (window.launch_combobox), cell, "text", 0, NULL);
  gtk_widget_show (window.launch_combobox);
  gtk_box_pack_end (GTK_BOX(hbox),window.launch_combobox,FALSE,FALSE,0);
  
  /* get selection */
  window.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (window.view_list));
  
  load_list();
  
  /* set 'default settings' in combobox and radiomenu - get index from list */
  match_string = gconf_client_get_string (client, GCONF_LIST_SETTINGS_STRING, NULL);
  
  valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (window.liststore), &iter);
  
  while ((valid == TRUE) && (exit == FALSE)) {
	  
	  gtk_tree_model_get (GTK_TREE_MODEL (window.liststore), &iter, 0, &next_string, -1);
	  
	  if (!g_ascii_strcasecmp (next_string, match_string))
	  {
		  /* radio menu */
		  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (g_ptr_array_index(item_array, index_item)), TRUE);
		  
		  /* combobox set */
		  gtk_combo_box_set_active_iter (GTK_COMBO_BOX(window.launch_combobox), &iter);
		  
		  exit = TRUE;
	  }
	  
	  valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(window.liststore), &iter);
	  index_item++;
	  
	  g_free (next_string);
  }
  
  /* if selected first element from radio menu */
  if (index_item == 1)
	  refresh_for_settings_selected (match_string);
  
  g_free (match_string);
  
  /* active all signals and unref objects */
  g_signal_connect (G_OBJECT (window.launch_combobox), "changed",
                      G_CALLBACK (on_changed_list_settings_combobox), NULL);
  
  g_signal_connect (G_OBJECT (window.selection), "changed",
                      G_CALLBACK (on_changed_list_settings_selection), NULL);
  
  glade_xml_signal_autoconnect(xml_glade);
  
  g_object_unref(G_OBJECT(window.liststore));
  g_object_unref(G_OBJECT(xml_glade));
}

/***********************/
/* New Settings Window */
/***********************/
void
on_cancel_new_settings_dialog_clicked (GtkButton *button, gpointer user_data)
{
	gtk_entry_set_text (GTK_ENTRY (dialog.entry), "");
	gtk_widget_hide (dialog.dialog);
}


/* foreach function to compare strings */
static gboolean
compare_strings (GtkTreeModel *model, GtkTreePath  *path, GtkTreeIter  *iter, gpointer user_data)
{
	gchar *tmp_settings_name;
	gchar *input_settings_name;
	
	input_settings_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog.entry)));
	g_strstrip (input_settings_name);
	
	gtk_tree_model_get (model, iter, 0, &tmp_settings_name, -1); 
	if (!g_ascii_strcasecmp (tmp_settings_name, input_settings_name))
	{
		GtkWidget *error_dialog;
		
		error_dialog = gtk_message_dialog_new (GTK_WINDOW(window.window),
       	                           GTK_DIALOG_DESTROY_WITH_PARENT,
           	                       GTK_MESSAGE_ERROR,
               	                   GTK_BUTTONS_OK,
                   	               _("You already have a 'settings' called \"%s\""),
                       	           input_settings_name);
		
		gtk_dialog_run (GTK_DIALOG (error_dialog));
		gtk_widget_destroy (error_dialog);
		
		status_dialog = TRUE;
	}
	
	g_free (tmp_settings_name);
	g_free (input_settings_name);
	
	return status_dialog;
}

void
on_create_new_settings_dialog_clicked (GtkButton *button, gpointer user_data)
{
	GtkTreeIter iter;
	
	status_dialog = FALSE;
		
	gtk_tree_model_foreach(GTK_TREE_MODEL(window.liststore), compare_strings, NULL);
	if (status_dialog == TRUE)
		return;
	
	/* load settings for 'base on' */
	g_free (settings.save_new_settings_name);
	g_free (settings.gconf_path_settings);
	
	gtk_combo_box_get_active_iter (GTK_COMBO_BOX(dialog.combobox), &iter);
	
	gtk_tree_model_get (GTK_TREE_MODEL(window.liststore), &iter, 0, &settings.save_new_settings_name, -1);
	
	settings.gconf_path_settings = g_strconcat("/apps/glsof/settings/", settings.save_new_settings_name, NULL);
	
	load_sets ();
	
	/* save settings with new name and update list settings */
	g_free (settings.save_new_settings_name);
	g_free (settings.gconf_path_settings);
	
	settings.save_new_settings_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog.entry)));
	g_strstrip (settings.save_new_settings_name);
	
	settings.gconf_path_settings = g_strconcat("/apps/glsof/settings/", settings.save_new_settings_name, NULL);

	gtk_list_store_append (GTK_LIST_STORE(window.liststore), &iter);
	
	gtk_list_store_set (GTK_LIST_STORE(window.liststore), &iter, 0, settings.save_new_settings_name,-1);
	
	window.list = g_slist_append (window.list, settings.save_new_settings_name);
	
	/* radio settings sub menu */
	add_item_radio ((gpointer) settings.save_new_settings_name);
	
	gconf_client_add_dir (client, settings.gconf_path_settings, GCONF_CLIENT_PRELOAD_NONE, NULL);
	
	change_title_settings_window();
	
	gtk_widget_show (settings.window);
	
	gconf_client_set_list (client, GCONF_LIST_SETTINGS_PATH, GCONF_VALUE_STRING, window.list, NULL);
	
	gtk_entry_set_text (GTK_ENTRY (dialog.entry),"");
	gtk_widget_hide (dialog.dialog);
	
	/* launch just made 'settings' */
	/*g_free (settings.save_new_settings_name);
	g_free (settings.gconf_path_settings);
	
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (g_ptr_array_index(item_array, item_array->len -1)), TRUE);*/
}

/* this little function is stolen from gnome-terminal 2.7.3 :) */
void
on_entry_new_settings_name_changed (GtkEditable *editable, gpointer data)
{
  gchar *name, *saved_name;

  saved_name = name = gtk_editable_get_chars (editable, 0, -1);

  while (*name != '\0' && g_ascii_isspace (*name))
    name++;

  gtk_widget_set_sensitive (dialog.create_button, *name != '\0' ? TRUE : FALSE);
  
  g_free (saved_name);
}

void
make_new_settings_dialog (void)
{
	GladeXML *xml_glade;
	GtkWidget *hbox;
	GtkCellRenderer *cell;
	
	xml_glade = glade_xml_new (GLADE_PATH"glsof_new_settings.glade", NULL, NULL);
	
	/* new settings window */
	dialog.dialog = glade_xml_get_widget (xml_glade, "glsof-new_settings_dialog");
	
	dialog.entry = glade_xml_get_widget (xml_glade, "new-settings-name-entry");
	dialog.create_button = glade_xml_get_widget (xml_glade, "new-settings-create-button");
	
	/* tree-model for combobox */
	dialog.combobox = gtk_combo_box_new_with_model (GTK_TREE_MODEL(window.liststore));
	g_object_unref(G_OBJECT(window.liststore));
  	cell = gtk_cell_renderer_text_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (dialog.combobox), cell, TRUE);
  	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (dialog.combobox), cell, "text", 0, NULL);
  	gtk_widget_show (dialog.combobox);
	gtk_combo_box_set_active (GTK_COMBO_BOX(dialog.combobox), 0);
	
	hbox = glade_xml_get_widget (xml_glade, "hbox9");
	gtk_box_pack_end (GTK_BOX(hbox),dialog.combobox,FALSE,FALSE,0);
	
	/* active all signals */
  	glade_xml_signal_autoconnect(xml_glade);
	g_object_unref(G_OBJECT(xml_glade));
}
