/*  main - program entry point.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>

#include "interface.h"
#include "preferences.h"
#include "settings.h"
#include "list_settings.h"
#include "list_int_addr.h"

#define CONF_GLOBAL_PREFIX	"/apps/glsof/global"
#define GCONF_PREFERENCES_PATH	"/apps/glsof/preferences/"

/* interface */
extern gchar *path;
extern main_window interface;	/* main window */

/* extern */
GConfClient *client;
guint timeout_splash_window = 0;


static gboolean
find_lsof_path (void)
{
  struct stat buf;
  gboolean show_error = TRUE;

  FILE *fp;
  gchar readbuf[BUFSIZ];
  gchar **tmp = NULL;
  guint i = 0, s = 0;

  if ((fp = popen ("whereis -b lsof", "r")))
    {

      if (fgets (readbuf, BUFSIZ, fp))
		  tmp = g_strsplit (readbuf, " ", -1);

      pclose (fp);

      while (readbuf[i])
	{
	  if (readbuf[i] == ' ') s++;
	  i++;
	}

      if (s)
	{
	  int k = 1;

	  while (k <= s)
	    {
	      i = 0;

	      while (tmp[k][i] != 0)
		{
		  if (tmp[k][i] == '\n')
		    tmp[k][i] = 0;
		  i++;
		}

	      stat (tmp[k], &buf);
	      if (!access (tmp[k], X_OK | F_OK) && !(buf.st_mode & S_IFDIR))
		{
		  path = (char *) g_strconcat (tmp[k], " -F +c0 ", NULL);
		  show_error = FALSE;
		  break;
		}
	      k++;
	    }
	}
    }

  g_strfreev (tmp);

  return show_error;
}

int
main (int argc, char *argv[])
{
  GError *err = NULL;
  gboolean tmp_check_show_splash_conf;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  if (find_lsof_path() == TRUE)
  {
	  GtkWidget *dialog;
		
	  dialog = gtk_message_dialog_new (GTK_WINDOW(interface.window),
      	                           GTK_DIALOG_DESTROY_WITH_PARENT,
           	                       GTK_MESSAGE_ERROR,
               	                   GTK_BUTTONS_OK,
                   	               _("Lsof was not found or you have right permissions.\n Please, check 'lsof' installation or permissions."));
		
	  gtk_dialog_run (GTK_DIALOG (dialog));
 	  gtk_widget_destroy (dialog);
		
	  exit (1);
  }

  gnome_program_init ("glsof", VERSION, LIBGNOMEUI_MODULE,
		      argc, argv, GNOME_PARAM_APP_DATADIR,
		      PACKAGE_DATA_DIR, NULL);
  glade_gnome_init ();

  gconf_init (argc, argv, NULL);
  client = gconf_client_get_default ();

  gconf_client_add_dir (client, CONF_GLOBAL_PREFIX,
			GCONF_CLIENT_PRELOAD_ONELEVEL, &err);

  if (err)
    {
      g_printerr (_("There was an error loading config from %s. (%s)\n"),
		  CONF_GLOBAL_PREFIX, err->message);
      g_error_free (err);
    }
	
  tmp_check_show_splash_conf = gconf_client_get_bool (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "noshowsplash"),NULL );
	
  /* make splash window */
  if (tmp_check_show_splash_conf == FALSE)
	  make_splash_window();

  /* make all windows */
  make_main_window();
  make_internet_addresses_window ();
  make_settings_window();
  make_preferences_window();
  make_list_settings_window();
  make_new_settings_dialog();
  
  /* close splash window */
  if (tmp_check_show_splash_conf == FALSE)
	  timeout_splash_window = gtk_timeout_add(1000, close_splash_window, NULL);
  
  /* show main window and run it */
  gtk_widget_show (interface.window);
  gtk_main ();
  
  return 0;
}
