/*  popupmenu - Make popupmenu and kill-pid option.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <errno.h>
#include <signal.h>

#include "interface.h"
#include "list_settings.h"
#include "settings.h"
#include "preferences.h"

/* interface */
extern main_window interface;	/* main window */

/* list_settings */
extern list_settings_window window;
extern GPtrArray *item_array;

/* preferences */
extern gboolean state_fields[12];
extern preferences_window preferences;

static guint pid;
static guint popup_item_counter;
static gboolean status_dialog;
static GtkWidget *popup_kill_button;

static void
kill_confirm_response (GtkWidget * dialog, int response_id,
		       GtkWindow * transient_parent)
{
	if (response_id == GTK_RESPONSE_ACCEPT)
		status_dialog = TRUE;
}

static gboolean
control_confirm (void)
{
	GtkWidget *dialog;

	status_dialog = FALSE;

/* kill: YES or NO */
	dialog = gtk_message_dialog_new (NULL,
					 GTK_DIALOG_DESTROY_WITH_PARENT,
					 GTK_MESSAGE_QUESTION,
					 GTK_BUTTONS_NONE,
					 _
					 ("Are you sure? It could be dangerous..."),
					 NULL);

	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
				GTK_STOCK_CANCEL,
				GTK_RESPONSE_REJECT,
				GTK_STOCK_DELETE, GTK_RESPONSE_ACCEPT, NULL);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
					 GTK_RESPONSE_ACCEPT);

	gtk_window_set_title (GTK_WINDOW (dialog), _("Kill PID"));
	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);

	g_signal_connect (G_OBJECT (dialog), "response",
			  G_CALLBACK (kill_confirm_response), NULL);

	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);

	return status_dialog;
}

static void
popup_kill_pid (GtkWidget * menuitem, gpointer userdata)
{
	int error;
	GtkWidget *dialog;

	if (control_confirm () == FALSE)
		return;

	error = kill (pid, 9);
	if (error == -1)
	{
		switch (errno)
		{
		case ESRCH:
			break;
		case EPERM:
			dialog = gtk_message_dialog_new (GTK_WINDOW
							 (userdata),
							 GTK_DIALOG_DESTROY_WITH_PARENT,
							 GTK_MESSAGE_ERROR,
							 GTK_BUTTONS_OK,
							 _
							 ("PID : %d \n\nYou do not have permission to end this process."),
							 pid);

			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);

			break;
		default:
			dialog = gtk_message_dialog_new (GTK_WINDOW
							 (userdata),
							 GTK_DIALOG_DESTROY_WITH_PARENT,
							 GTK_MESSAGE_ERROR,
							 GTK_BUTTONS_OK,
							 _
							 ("An error occured while killing the process."),
							 NULL);
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);


		}
	}
}

/* button on main window for kill pid */
void
on_killpid_button_clicked (GtkButton * button, gpointer user_data)
{
	popup_kill_pid (NULL, (gpointer) interface.window);
}


/* settings menu */
static void
popup_menuitem_settings_response (GtkCheckMenuItem *
				  checkmenuitem, gpointer userdata)
{
	if (!gtk_check_menu_item_get_active (checkmenuitem))
		return;

	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM
					(g_ptr_array_index
					 (item_array,
					  (guint) userdata)), TRUE);
}

static void
popup_radio_handler_list (gpointer data, gpointer userdata)
{
	GtkWidget *item;
	static GSList *group_radio;

	/* radio settings sub menu */
	item = gtk_radio_menu_item_new_with_label (group_radio, data);

	group_radio =
		gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));

	gtk_menu_shell_append (GTK_MENU_SHELL (userdata), item);

	if (gtk_check_menu_item_get_active
	    (GTK_CHECK_MENU_ITEM
	     (g_ptr_array_index (item_array, popup_item_counter))) == TRUE)
		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (item), TRUE);

	/* Launch 'settings' when the menuitem is selected */
	g_signal_connect (G_OBJECT (item), "toggled",
			  G_CALLBACK
			  (popup_menuitem_settings_response),
			  (gpointer) popup_item_counter);

	popup_item_counter++;
}

/* view menu */
static void
popup_menuitem_view_response (GtkCheckMenuItem *
			      checkmenuitem, gpointer userdata)
{
	gboolean state;

	state = gtk_check_menu_item_get_active
		(GTK_CHECK_MENU_ITEM (checkmenuitem));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (preferences.
				       checkbutton_fields[(guint) userdata]),
				      state);
}

static void
popup_check_list (GtkWidget * menu)
{
	guint i = 1;
	const gchar *column_name[] = {
		"PID", "PGID", "PPID", "USER", "FD", "TYPE",
		"DEVICE", "SIZEOFF", "NLINK", "INODE"
	};

	for (; i < 11; i++)
	{
		GtkWidget *item;

		item = gtk_check_menu_item_new_with_label
			(column_name[i - 1]);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

		gtk_check_menu_item_set_active
			(GTK_CHECK_MENU_ITEM (item), state_fields[i]);

		/* Launch 'settings' when the menuitem is selected */
		g_signal_connect (G_OBJECT (item), "toggled",
				  G_CALLBACK
				  (popup_menuitem_view_response),
				  (gpointer) i - 1);
	}
}


/*******************/
/* Make popup menu */
/*******************/
void
view_popup_menu (GtkWidget * treeview,
		 GdkEventButton * event, gpointer userdata)
{
	GtkWidget *menu;
	GtkWidget *menuitem;
	GtkWidget *separator;
	GtkWidget *new_settings_radio_menu;


	/* popup menu */
	menu = gtk_menu_new ();

	/* 'change settings' menu */
	menuitem = gtk_menu_item_new_with_label (_("Change settings"));

	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);

	new_settings_radio_menu = gtk_menu_new ();

	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
				   new_settings_radio_menu);

	popup_item_counter = 0;

	g_slist_foreach (window.list,
			 (GFunc) popup_radio_handler_list,
			 (gpointer) new_settings_radio_menu);

	/* 'current settings' */
	menuitem = gtk_menu_item_new_with_label (_("Current settings..."));

	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);

	g_signal_connect (menuitem, "activate",
			  (GCallback) on_settings_clicked, NULL);

	/* separator */
	separator = gtk_separator_menu_item_new ();
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), separator);

	/* 'view' menu */
	menuitem = gtk_menu_item_new_with_label (_("View columns"));

	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);

	new_settings_radio_menu = gtk_menu_new ();

	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
				   new_settings_radio_menu);

	popup_check_list (new_settings_radio_menu);

	/* separator */
	separator = gtk_separator_menu_item_new ();
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), separator);

	/* PID menu */
	popup_kill_button = gtk_menu_item_new_with_label (_("Kill PID"));
	gtk_widget_set_sensitive (popup_kill_button, FALSE);

	gtk_menu_shell_append (GTK_MENU_SHELL (menu), popup_kill_button);

	g_signal_connect (popup_kill_button, "activate",
			  (GCallback) popup_kill_pid, NULL);

	gtk_widget_show_all (menu);

	/* Note: event can be NULL here when called from view_onPopupMenu;
	 *  gdk_event_get_time() accepts a NULL argument */
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL,
			NULL,
			(event != NULL) ? event->button : 0,
			gdk_event_get_time ((GdkEvent *) event));
}


gboolean
view_onButtonPressed (GtkWidget * treeview,
		      GdkEventButton * event, gpointer userdata)
{
	/* single click with the right or left mouse button? */
	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
		view_popup_menu (treeview, event, userdata);

	return FALSE;		/* we did not handle this */
}


gboolean
view_onPopupMenu (GtkWidget * treeview, gpointer userdata)
{
	view_popup_menu (treeview, NULL, userdata);

	return TRUE;		/* we handled this */
}


void
catch_pid (GtkTreeSelection * treeselection, gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *strpid;

	if (gtk_tree_selection_get_selected (treeselection, &model, &iter))
	{
		guint colpid = 1;
		
		if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (interface.item_layout1)) == FALSE)
			colpid = 0;
		
		/* catch the pid */
		gtk_tree_model_get (model, &iter, colpid, &strpid, -1);
		pid = g_ascii_strtod (strpid, NULL);
		g_free (strpid);

		gtk_widget_set_sensitive (interface.killpid_button, TRUE);
		if (popup_kill_button != NULL)
			gtk_widget_set_sensitive (popup_kill_button, TRUE);
	}
}
