/*  preferences - Make preferences window and save view-columns-state and other options.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>

#include "layouts.h"
#include "preferences.h"
#include "interface.h"

#define GCONF_PREFERENCES_PATH	"/apps/glsof/preferences/"

/* interface */
extern gchar *path;
extern main_window interface;

/* layouts */
extern GtkWidget *view;	/* element for main table */

/* refresh */
extern GPtrArray *list_layout2_table;

/* main */
extern GConfClient *client;

/* extern */
preferences_window preferences;
gboolean state_fields[12];

static const gchar *key[] = {
	  "pid", "pgid", "ppid", "user", "fd", "type", "device", "sizeoff", "nlink", "inode"};


/* set status columns at start */
static void 
set_status_columns(void)
{
  guint i,j;
  GtkTreeViewColumn *column;
  layout2_elem *elem;

	
  /* hide & show columns for single list view */
  for (i = 1; i < 11; i++)
    {
	  column = gtk_tree_view_get_column (GTK_TREE_VIEW(view),i);
  	  gtk_tree_view_column_set_visible (column, state_fields[i]);
    }
	
  /* hide & show columns for separated list view */
  for (i = 0; i<list_layout2_table->len; i++)
  {
	  elem = g_ptr_array_index (list_layout2_table,i);
	  
	  for (j = 0; j < 10; j++)
	  {
		  column = gtk_tree_view_get_column (GTK_TREE_VIEW(elem->view),j);
		  gtk_tree_view_column_set_visible (column, state_fields[j+1]);
	  }
  }
}

/* columns hide & view */
void
show_hide_columns (GtkToggleButton *togglebutton, gpointer user_data)
{
  guint i,j;
  GtkTreeViewColumn *column;
  layout2_elem *elem;
	
  i = GPOINTER_TO_INT (user_data);
	
  state_fields[i] = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (togglebutton));
  gconf_client_set_bool (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, key[i-1]), state_fields[i],NULL);
	
  column = gtk_tree_view_get_column (GTK_TREE_VIEW(view),i);
  gtk_tree_view_column_set_visible (column, state_fields[i]);
	
  for (j = 0; j<list_layout2_table->len; j++)
  {
	  elem = g_ptr_array_index (list_layout2_table,j);
	  column = gtk_tree_view_get_column (GTK_TREE_VIEW(elem->view),i-1);
	  gtk_tree_view_column_set_visible (column, state_fields[i]);
  }
}


void 
save_size_and_position_main_window (void)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences.sets_check_conf)))
    {
	  gint x, y, w, h;
	  
      gtk_window_get_position (GTK_WINDOW (interface.window), &x, &y);
      gtk_window_get_size (GTK_WINDOW (interface.window), &w, &h);

      gconf_client_set_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "windowpositionx"), x,NULL );
	  gconf_client_set_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "windowpositiony"), y,NULL );
	  gconf_client_set_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "window-width"), w,NULL );
	  gconf_client_set_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "window-height"), h,NULL );
    }
}


void 
get_size_and_position_main_window (void)
{
	gint x, y, w, h;
	
	x = gconf_client_get_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "windowpositionx"), NULL);
	y = gconf_client_get_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "windowpositiony"), NULL);
	w = gconf_client_get_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "window-width"), NULL);
	h = gconf_client_get_int (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "window-height"), NULL);
	
	gtk_window_move (GTK_WINDOW (interface.window), x, y);
    gtk_window_resize (GTK_WINDOW (interface.window), w, h);
}


/* Configuration Window */
void on_main_cb1_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	gconf_client_set_bool (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "keepwindowsize"), gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences.sets_check_conf)) ,NULL );
}

void on_main_cb2_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	gconf_client_set_bool (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "noshowsplash"), gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (preferences.sets_check_show_splash_conf)) ,NULL );
}

void
on_preferences_clicked (GtkButton * button, gpointer user_data)
{
	gtk_widget_show (preferences.window);
}

void
on_close_clicked (GtkButton * button, gpointer user_data)
{
	gtk_widget_hide (preferences.window);
}

void
make_preferences_window (void)
{
  GladeXML *xml_glade;
  guint i;


  xml_glade = glade_xml_new (GLADE_PATH"glsof_preferences.glade", NULL, NULL);
	
  preferences.window = glade_xml_get_widget (xml_glade, "glsof-preferences");
  preferences.sets_entry_conf = glade_xml_get_widget (xml_glade, "fileentry1");
  preferences.sets_check_conf = glade_xml_get_widget (xml_glade, "main_cb1");
  preferences.sets_check_show_splash_conf = glade_xml_get_widget (xml_glade, "main_cb2");
	
  preferences.checkbutton_fields[0] = glade_xml_get_widget (xml_glade, "checkbutton1");
  preferences.checkbutton_fields[1] = glade_xml_get_widget (xml_glade, "checkbutton2");
  preferences.checkbutton_fields[2] = glade_xml_get_widget (xml_glade, "checkbutton3");
  preferences.checkbutton_fields[3] = glade_xml_get_widget (xml_glade, "checkbutton4");
  preferences.checkbutton_fields[4] = glade_xml_get_widget (xml_glade, "checkbutton5");
  preferences.checkbutton_fields[5] = glade_xml_get_widget (xml_glade, "checkbutton6");
  preferences.checkbutton_fields[6] = glade_xml_get_widget (xml_glade, "checkbutton7");
  preferences.checkbutton_fields[7] = glade_xml_get_widget (xml_glade, "checkbutton8");
  preferences.checkbutton_fields[8] = glade_xml_get_widget (xml_glade, "checkbutton9");
  preferences.checkbutton_fields[9] = glade_xml_get_widget (xml_glade, "checkbutton10");
	
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences.sets_check_conf),
  	gconf_client_get_bool (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "keepwindowsize"), NULL));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preferences.sets_check_show_splash_conf), gconf_client_get_bool (client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "noshowsplash"), NULL));
				
  for (i=1; i<11; i++)
  {
	  state_fields [i] = gconf_client_get_bool (client, 
	  gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, key[i-1]), NULL);
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(preferences.checkbutton_fields[i-1]),state_fields[i]);
	  
	  g_signal_connect ((gpointer) preferences.checkbutton_fields[i-1], "clicked",
		    G_CALLBACK (show_hide_columns), GINT_TO_POINTER (i));
  }
  
  state_fields [0] = TRUE;
  state_fields [11] = TRUE;
  
  set_status_columns();
  
  glade_xml_signal_autoconnect(xml_glade);
  g_object_unref(G_OBJECT(xml_glade));
}
