/*  search - search datas in the list.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "interface.h"

/* interface */
extern main_window interface;

/* layouts */

/* layouts 1 */
extern GtkListStore *command_info_table;
extern GtkTreeSelection *selection_single_list;
extern GtkWidget *view;

/* layout 2 */
extern GtkTreeSelection *selection_left_command_column;

/* refresh */
extern guint append_num;

static gboolean exit_loop = FALSE;

/* extern */
gchar *name = NULL;
gchar *name_copy = NULL;


void
on_search_entry_changed (GtkEditable * editable, gpointer data)
{
	gboolean state = FALSE;

	name = gtk_editable_get_chars (editable, 0, -1);

	if (*name != '\0')
		state = TRUE;

	gtk_widget_set_sensitive (interface.combox, state);
	gtk_widget_set_sensitive (interface.search_button, state);
}

/* dialog search failed */
void
dialog_search_failed (void)
{
	GtkWidget *dialog;

	dialog = gtk_message_dialog_new (GTK_WINDOW (interface.window),
					 GTK_DIALOG_DESTROY_WITH_PARENT,
					 GTK_MESSAGE_ERROR,
					 GTK_BUTTONS_OK,
					 _("%s could not be found."), name);

	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);

	return;
}

/* dialog restart */
static void
restart_confirm_response (GtkWidget * dialog, int response_id,
			  GtkWindow * transient_parent)
{
	if (response_id == GTK_RESPONSE_NO)
		exit_loop = TRUE;
}


static gboolean
dialog_restart (void)
{
	GtkWidget *dialog;

	/* restart: YES or NO */
	dialog = gtk_message_dialog_new (GTK_WINDOW (interface.window),
					 GTK_DIALOG_DESTROY_WITH_PARENT,
					 GTK_MESSAGE_QUESTION,
					 GTK_BUTTONS_NONE, 
					 _("%s was not found. Wrap search around the list?"),
					 name);

	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
				GTK_STOCK_NO,
				GTK_RESPONSE_NO,
				GTK_STOCK_YES, GTK_RESPONSE_YES, NULL);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
					 GTK_RESPONSE_ACCEPT);

	gtk_window_set_title (GTK_WINDOW (dialog), _("Searching..."));
	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);

	g_signal_connect (G_OBJECT (dialog), "response",
			  G_CALLBACK (restart_confirm_response), NULL);

	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);

	return TRUE;
}


/* dialog no found lines */
void
dialog_no_found_lines (void)
{
	GtkWidget *dialog;

	dialog = gtk_message_dialog_new (GTK_WINDOW (interface.window),
					 GTK_DIALOG_DESTROY_WITH_PARENT,
					 GTK_MESSAGE_ERROR,
					 GTK_BUTTONS_OK,
					 _("The list is empty."));

	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);

	return;
}


void
on_search_button_clicked (GtkButton * button, gpointer user_data)
{
	if (append_num == 0)
	{
		dialog_no_found_lines ();
		return;
	}

	else
	{
		guint index;
		static guint index_copy = 0;
		static guint index_row = 0;
		static gboolean found_for_this_name = FALSE;
		static GtkTreeIter iter;	/* single layer */

		index = gtk_combo_box_get_active (GTK_COMBO_BOX
						  (interface.combox));

		if (name_copy == NULL)
			name_copy = g_strdup (name);
		else if (g_ascii_strcasecmp (name, name_copy) != 0)
		{
			g_free (name_copy);
			name_copy = g_strdup (name);

			/* restart conditions */
			index_row = 0;
			found_for_this_name = FALSE;
		}
		else if (index != index_copy)
		{
			index_copy = index;
			
			/* restart conditions */
			index_row = 0;
			found_for_this_name = FALSE;
		}

		while (exit_loop == FALSE)
		{
			gboolean valid;
			gchar *column[12];
			guint i;

			if (index_row == 0)
				valid = gtk_tree_model_get_iter_first
					(GTK_TREE_MODEL (command_info_table),
					 &iter);
			else
				valid = gtk_tree_model_iter_next
					(GTK_TREE_MODEL (command_info_table),
					 &iter);

			if (valid == TRUE)
			{
				/* get single row for multiple columns */
				if (index == 0)
				{
					/* SINGLE LAYER */
					gtk_tree_model_get (GTK_TREE_MODEL
							    (command_info_table),
							    &iter, 0,
							    &column[0], 1,
							    &column[1], 2,
							    &column[2], 3,
							    &column[3], 4,
							    &column[4], 5,
							    &column[5], 6,
							    &column[6], 7,
							    &column[7], 8,
							    &column[8], 9,
							    &column[9], 10,
							    &column[10], 11,
							    &column[11], -1);

					for (i = 0; i < 12; i++)
					{
						if (g_strrstr (column[i], name))	/* matched */
						{
							GtkTreePath *node =
								gtk_tree_model_get_path
								(GTK_TREE_MODEL
								 (command_info_table),
								 &iter);
							gtk_tree_view_set_cursor
								(GTK_TREE_VIEW
								 (view), node,
								 NULL, FALSE);
							gtk_tree_path_free
								(node);
							exit_loop = TRUE;
							found_for_this_name =
								TRUE;

							/* DOUBLE LAYERS */
							//search_in_double_layer(column [0]);

							break;
						}
					}

					for (i = 0; i < 12; i++)
						g_free (column[i]);
				}
				/* get row for single column */
				else
				{
					i = index - 1;
					gtk_tree_model_get (GTK_TREE_MODEL
							    (command_info_table),
							    &iter, i,
							    &column[i], -1);

					if (g_strrstr (column[i], name))
					{
						GtkTreePath *node =
							gtk_tree_model_get_path
							(GTK_TREE_MODEL
							 (command_info_table),
							 &iter);
						gtk_tree_view_set_cursor
							(GTK_TREE_VIEW (view),
							 node, NULL, FALSE);
						gtk_tree_path_free (node);
						exit_loop = TRUE;
						found_for_this_name = TRUE;
					}

					g_free (column[i]);
				}

				index_row++;
			}
			else
			{
				/* if one row is selected launch 'restart' dialog */
				if (found_for_this_name == TRUE)
				{
					dialog_restart ();
					found_for_this_name = FALSE;
				}
				else
				{
					dialog_search_failed ();
					exit_loop = TRUE;
				}

				index_row = 0;
			}

		}		/* _while */

		exit_loop = FALSE;
	}
}
